module Borges

class HTML

  def initialize(stream = "")
    @stream = stream
  end
  
  def print(text)
    @stream << text.to_s
  end
  
  def open_tag(name, attrs={})
    print "<#{name}"
    attrs.each do |k, v|
      case v
      when true
        print " #{k}"
      when false
      else
        print " #{k}='#{v}'"
      end
    end
    print ">"
  end
  
  def close_tag(name)
    print "</#{name}>"
  end
  
  def simple_tag(name, text, attrs={})
    open_tag(name, attrs)
    print text
    close_tag(name)
  end
  
  def tag(name, attrs={})
    open_tag(name, attrs)
    yield
    close_tag(name)
  end
  
  def space() print "&nbsp;" end
  def break() print "<br>" end
  def heading(text, n=1, attrs={}) simple_tag("h#{n}", text, attrs) end

  def self.tags(*tag_names)
    tag_names.each do |tag|
      module_eval %!
        def #{tag}(attrs={})
          tag("#{tag}", attrs) {yield}
        end
      !
    end
  end
  
  tags :p, :table, :td, :tr, :div, :span
end

end

