require 'borges/Renderer'

module Borges

class Page

  attr_accessor :title

  ##
  # Allow a Page to be used by Application
  #
  #   class MyPage < Page; ...; end
  #   Borges.register_application('mypage', MyPage)

  def self.call
    self.new.go
  end
  
  ##
  # Run an instance of the application.  Will capture the Renderer object on
  # the stack in the continuation.

  def go
    callcc { |@answer| response_loop }
  end
  
  ##
  # Event loop for handling requests

  def response_loop
    loop { render { |r| render_on(r) } }
  end
  
  ##
  # Render this page on the Renderer +r+

  def render_on(r, embeded = false)
    unless embeded then
      r.open_tag("html")
      r.open_tag("head")
      r.open_tag("title")
      r.print(@title)
      r.close_tag("title")
      on_head(r)
      r.close_tag("head")
      r.open_tag("body")
    end

    on_body(r)

    unless embeded then
      r.close_tag("body")
      r.close_tag("html")
    end
  end

  ##
  # Render the contents of the <head> element on +r+
  # Optionally implemented by the subclass.

  def on_head(r)
  end
  
  ##
  # Render the contents of the <body> element on +r+
  # Must be implemented by the subclass.

  def on_body(r)
    raise "Subclass responsibility"
  end

  def answer(return_val=nil)
    @answer.call(return_val)
  end
end

end

