require 'test/unit'
require 'Utilities'

class TC_LRUCache < Test::Unit::TestCase

  ##
  # Store must give an index from which an entry can be retrieved

  def test_store
    cache = LRUCache.new(1)

    a_index = cache.store(:a)

    assert_equal(:a, cache[a_index], "Invalid index given by #store")
  end

  ##
  # New entry must displace an older one if cache is out of space

  def test_retrieve
    cache = LRUCache.new(1)

    a_index = cache.store(:a)
    b_index = cache.store(:b)

    assert_equal(nil, cache[a_index], "Older entry not thrown out")
    assert_equal(:b, cache[b_index])
  end

  ##
  # Give a choice of two entries with equal weight, throw out the oldest

  def test_throwaway_oldest_1
    cache = LRUCache.new(2)

    a_index = cache.store(:a)
    b_index = cache.store(:b)
    c_index = cache.store(:c)

    assert_equal(nil, cache[a_index])
    assert_equal(:b, cache[b_index])
    assert_equal(:c, cache[c_index])
  end

  ##
  # Always throw out the oldest entry

  def test_throwaway_oldest_2
    cache = LRUCache.new(2)

    a_index = cache.store(:a)
    cache[a_index]
    b_index = cache.store(:b)
    c_index = cache.store(:c)

    assert_equal(:a, cache[a_index])
    assert_equal(nil, cache[b_index])
    assert_equal(:c, cache[c_index])
  end
end

