class Cache < Hash
  def store(item)
    @count ||= 0
    @count += 1
    self[@count.to_s] = item
    @count.to_s
  end
end

class LRUCache
  include Enumerable

  def initialize(max = 50)
    @hash = Hash.new
    @max = max
    @used = 0
    @count = 0
    @cur_access = 0
  end

  ##
  # Store an item, if out of space throw away the least recently used, oldest
  # item

  def store(item)
    @count += 1
    @used += 1

    if @used > @max then
      hash_arr = @hash.to_a.sort do |a, b|
        a[1][0] <=> b[1][0]
      end

      if hash_arr.last[1][0].zero? then
        hash_arr.delete_if do |x|
          x[1][0] != 0
        end

        hash_arr.sort! do |a, b|
          a[0] <=> b[0]
        end
      end

      @hash.delete(hash_arr.first[0])

      @used = @max
    end

    @hash[@count.to_s] = [0, item]
    @count.to_s
  end

  ##
  # Retrieve an item and bump its access index

  def [](item)
    item = @hash[item]
    unless item.nil? then
      @cur_access += 1
      item[0] = @cur_access
      return item[1]

    else
      return nil

    end
  end

  ##
  # Iterate over the items in the cache.  (For debugging)

  def each(&block)
    @hash.each do |k, v|
      yield v[1]
    end
  end

  ##
  # Number of items stored

  def size() @used end
end

module Borges

##
# Base class for Request

class Request
  include Enumerable

  attr_reader :application_key
  attr_reader :session_key
  attr_reader :action_key

  attr_reader :url
  attr_reader :fields

  def initialize(url, fields)
    @url = url
    @fields = fields

    # application / session_id / action_id
    @url =~ %r!^#{Borges::base_url}(.*?)/?(?:(\d+)(?:/(\d+))?)?$!
    @application_key = $1
    @session_key = $2
    @action_key = $3
  end

  def each
    @fields.each{|a| yield a}
  end

  def [](key)
    @fields[key]
  end
end

class Response

  attr_accessor :body, :headers, :status
  attr_accessor :session_key, :action_key, :application_key
  
  def initialize
    @body = ""
    @headers = {}
    @status = 200
    @headers['content-type'] = 'text/html'
  end
  
  def <<(text)
    @body << text
  end
  
  def not_found(msg, request)
    @status = 404
    self << "<h1>#{msg}</h1>"
  end

  def error(msg, request)
    @status = 500
    self << "<h1 style=\"color:red\">#{msg}</h1>"
  end

  def redirect(status, location)
    @status = status
    @headers['Location'] = location
  end
  
  def action_url(key = @action_key)
    session_url + "/" + key
  end
  
  def session_url
    application_url + "/" + @session_key
  end
  
  def application_url
    Borges.base_url + @application_key
  end
end

##
# Fetch the base url

def self.base_url
  @@base_url
end

##
# Set the base url, must begin and end with a '/'

def self.base_url=(url)
  unless url[0].chr == '/' and url[-1].chr == '/' then
    raise "Invalid base url: ``#{url}'', must begin and end with a '/'"
  end

  @@base_url = url
end

@@base_url = "/"

end

