#!/usr/bin/env ruby -w

require 'borges'
require 'borges/Page'

include Borges

##
# A simple view of the status of Borges, displays installed applications,
# active sessions, and active continuations

module Borges

class Status < Page

  ##
  # Build the page

  def initialize
    @title = 'Borges Status'
  end

  def on_body(r)
    num_sessions = 0

    apps = Borges::Application::Apps
    apps.each do |app_name, app|
      num_sessions += app.sessions.size
    end

    r.open_tag "p"
    r.print "There are #{num_sessions} active sessions"
    r.print "\n"

    r.open_tag "ul"
    r.print "\n"

    apps.sort.each do |app_name, app|
      r.open_tag "li"
      r.print "<a href=\"/#{app_name}\">/#{app_name}</a>: #{app.sessions.size} Sessions\n"

      r.open_tag "ul"
      r.print "\n"

      app.sessions.each do |session|
        r.open_tag "li"
        r.print "Continuations: #{session.continuations.size}"
        r.print "\n"
      end

      r.close_tag "ul"
      r.print "\n"
    end

    r.close_tag("ul")
    r.print "\n"

    r.anchor("Refresh") { } # do nothing
  end

end # class Status

end # module Borges

Borges::register_application('status', Borges::Status)

if $0 == __FILE__ then
  require 'borges/Webrick'

  Borges::WebrickConnection.start
end

